@@ -3,9 +3,9 @@
 #' @param x a vector
 #' @keywords internal
 
-check_for_two_levels <- function(x){
+check_for_two_levels <- function(x) {
   xunique <- sort(unique(x))
-  if(length(xunique) == 2 && all(xunique != c(0, 1))){
+  if (length(xunique) == 2 && all(xunique != c(0, 1))) {
     warning(sprintf("x has two levels, but they are not code as 0/1. SMD is being computed based on the values %s.", paste(xunique, collapse = ",")))
   }
 }
@@ -20,8 +20,9 @@ check_for_two_levels <- function(x){
 #' @param ref the index of the reference element
 #' @keywords internal
 
-lapplyFUNpairwise <- function(x, f, ref){
-  if(sum(is.na(x)) >= 1) stop("x contians NA value")
-  lapply(x[-ref], FUN = function(y) { f(x[[ref]], y) })
+lapplyFUNpairwise <- function(x, f, ref) {
+  if (sum(is.na(x)) >= 1) stop("x contains NA value")
+  lapply(x[-ref], FUN = function(y) {
+    f(x[[ref]], y)
+  })
 }
-
